unit modSpectrum;
{ /*******************************************************************************
'   modSpectrum.pas within DelphiSpec.dpr
'
'   Routines for emulating the spectrum hardware; displaying the
'   video memory (0x4000 - 0x5AFF), reading the keyboard (port
'   0xFE), and displaying the border colour (out (xxFE),x)
'
'   Author: Chris Cowley <ccowley@grok.co.uk>
'
'   Copyright (C)1999  Grok Developments Ltd.
'   http://www.grok.co.uk/
'
'   Translation to Delphi Object Pascal by
'           Jari Korhonen <jarit.korhonen@luukku.com>
'
'   This program is free software; you can redistribute it and/or
'   modify it under the terms of the GNU General Public License
'   as published by the Free Software Foundation; either version 2
'   of the License, or (at your option) any later version.
'   This program is distributed in the hope that it will be useful,
'   but WITHOUT ANY WARRANTY; without even the implied warranty of
'   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'   GNU General Public License for more details.
'
'   You should have received a copy of the GNU General Public License
'   along with this program; if not, write to the Free Software
'   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' *******************************************************************************/}


interface

uses windows, classes, sysutils;


var
glNewBorder : integer;

function inb(port : integer) : integer;

procedure outb(port : integer; outbyte : integer);
procedure plot(addr : integer);
procedure refreshFlashChars;
procedure screenPaint;


implementation

uses modMain, modAY8912, spectrum;


procedure plot(addr : integer);
    var lne : integer; i : integer; X : integer;
begin
    If addr < 22528 Then
    begin
        // Alter a pixel
        lne := ((addr shr 8) And $7) Or
                  ((addr shr 2) And $38) Or
                  ((addr shr 5) And $C0);
        ScrnLines[lne, 32] := True;
        ScrnLines[lne, addr And 31] := True;
    end
    Else
    begin
        // Alter an attribute
        lne := ((addr - 22528) shr 5);
        lne := lne shl 3;
        X := addr Mod 32;
        For i := lne To lne + 7 do
        begin
            ScrnLines[i, 32] := True;
            ScrnLines[i, X] := True
        end;
    End;
    ScrnNeedRepaint := True;
End;


Function inb(port : integer) : integer;
    var res : integer;
begin
    res := $FF;
    If (port And $1) = 0 Then
    begin
        If (port And $8000) = 0 Then res := res And keyB_SPC;
        If (port And $4000) = 0 Then res := res And keyH_ENT;
        If (port And $2000) = 0 Then res := res And keyY_P;
        If (port And $1000) = 0 Then res := res And key6_0;
        If (port And $800) = 0 Then res := res And key1_5;
        If (port And $400) = 0 Then res := res And keyQ_T;
        If (port And $200) = 0 Then res := res And keyA_G;
        If (port And $100) = 0 Then res := res And keyCAPS_V;
        inb := res;
    end
{$IFDEF USE_SOUND}
    Else If port = $FFFD Then
        inb := AYPSG.Regs[glSoundRegister]
{$ENDIF}
    Else
        inb := 0;
End;

procedure outb(port : integer; outbyte : integer);
begin
    If (port And $1) = 0 Then
    begin
        If frmMainWnd.borderColor <> (outbyte And $7) Then
            frmMainWnd.borderColor := (outbyte And $7);
{$IFDEF USE_SOUND}
        If ((outbyte And 16) = 16) Then
        begin
{$IFDEF USE_DEBUGGING}
            ReportDebug('glBeeperVal=31',0,True);
{$ENDIF}
            glBeeperVal := 31
        end
        Else
            glBeeperVal := 0;
{$ENDIF}
    end
    Else If glMemPagingType <> 0 Then
    begin
        // 128/+2 memory page operation
        If port = $7FFD Then
        begin
            // RAM page
            glPageAt[3] := (outbyte And 7);
            // Screen page
            If ((outbyte And 8) = 8) Then
            begin
                If glUseScreen = 5 Then
                begin
                    glUseScreen := 7;
                    ForceRepaint;
                End;
            end
            Else
            begin
                If glUseScreen = 7 Then
                begin
                    glUseScreen := 5;
                    ForceRepaint;
                End;
            End;

            // ROM
            If ((outbyte And 16) = 16) Then
            begin
                glPageAt[0] := 9;
                glPageAt[4] := 9;
            end
            Else
            begin
                glPageAt[0] := 8;
                glPageAt[4] := 8;
            End;

            If ((outbyte And 32) = 32) Then glMemPagingType := 0;
            glLastOut7FFD := outbyte;
        end
{$IFDEF USE_SOUND}
        Else If port = $FFFD Then
            glSoundRegister := outbyte And $F
        Else If port = $BFFD Then
            AYWriteReg(glSoundRegister, outbyte)
        Else If port = $BEFD Then
            AYWriteReg(glSoundRegister, outbyte);
        //ElseIf port = $1FFD Then
        // +2A/+3 special paging mode
{$ENDIF}
    End;
End;


procedure refreshFlashChars;
    var addr : integer; lne : integer; i : integer;
begin

    bFlashInverse := Not (bFlashInverse);

    For addr := 6144 To 6911 do
    begin
        If (gRAMPage[5, addr] And 128) = 128 Then
        begin
            lne := ((addr - 6144) shr 5);
            lne := lne shl 3;
            For i := lne To lne + 7 do
            begin
                ScrnLines[i, 32] := True;
                ScrnLines[i, addr And 31] := True;
            end;
            ScrnNeedRepaint := True;
        End;
    end;
End;



procedure screenPaint1X;
    var
    lne : integer; X : integer;
    sbyte : integer; abyte : integer; fcolor : integer; bcolor : integer;
    lXBit : integer; lXOffset : integer;
    charstart : integer;
    P:^TByteArray;
begin

  If ScrnNeedRepaint = False Then
    Exit;

  For lne := 0 To 191 do
  begin
      If ScrnLines[lne, 32] = True Then
      begin
          P :=frmMainWnd.DXPaintBoxSpecScreen.DIB.ScanLine[lne];
          For X := 0 To 31 do
          begin
              If ScrnLines[lne, X] = True Then
              begin
                  sbyte := gRAMPage[glUseScreen, glScreenMem[lne, X]];
                  abyte := gRAMPage[glUseScreen, (6144 + (lne shr 3) shl 5 + X)];

                  charStart := X shl 3;
                  If (abyte And 64) = 64 Then
                  begin
                     fcolor := (abyte And 7) + 8;
                     bcolor := ((abyte And 56) shr 3) + 8;
                  end
                  Else
                  begin
                     fcolor := (abyte And 7);
                     bcolor := ((abyte And 56) shr 3);
                  End;
                  If ((abyte And 128) = 128) And (bFlashInverse) Then
                  begin
                      // Swap fore- and back-colours
                      lXBit := fcolor;
                      fcolor := bcolor;
                      bcolor := lXBit;
                  End;

                  lXBit := 128;
                  lXOffset := 0;
                  repeat
                      If (sbyte And lXBit) = lXBit Then
                          P[charstart+lXOffset] := fcolor
                      Else
                          P[charstart+lXOffset] := bcolor;
                      lXBit := lXBit shr 1;
                      lXOffset := lXOffset + 1;
                  Until lXBit = 0;
              End;
              ScrnLines[lne, X] := False;
          end;
          ScrnLines[lne, 32] := False;
      End;
  end;
  frmMainWnd.DXPaintBoxSpecScreen.Paint;
  ScrnNeedRepaint := False;
End;


procedure screenPaint2X;
    var
    lne : integer; X : integer;
    sbyte : integer; abyte : integer; fcolor : integer; bcolor : integer;
    lXBit : integer; lXOffset : integer;
    charstart : integer;
    P1:^TByteArray;
    P2:^TByteArray;
begin

  If ScrnNeedRepaint = False Then
    Exit;

  For lne := 0 To 191 do
  begin
      If ScrnLines[lne, 32] = True Then
      begin
          P1:=frmMainWnd.DXPaintBoxSpecScreen.DIB.ScanLine[lne shl 1];
          P2:=frmMainWnd.DXPaintBoxSpecScreen.DIB.ScanLine[lne shl 1 + 1];
          For X := 0 To 31 do
          begin
              If ScrnLines[lne, X] = True Then
              begin
                  sbyte := gRAMPage[glUseScreen, glScreenMem[lne, X]];
                  abyte := gRAMPage[glUseScreen, (6144 + (lne shr 3) shl 5 + X)];

                  charStart := X shl 4;
                  If (abyte And 64) = 64 Then
                  begin
                     fcolor := (abyte And 7) + 8;
                     bcolor := ((abyte And 56) shr 3) + 8;
                  end
                  Else
                  begin
                     fcolor := (abyte And 7);
                     bcolor := ((abyte And 56) shr 3);
                  End;
                  If ((abyte And 128) = 128) And (bFlashInverse) Then
                  begin
                      // Swap fore- and back-colours
                      lXBit := fcolor;
                      fcolor := bcolor;
                      bcolor := lXBit;
                  End;

                  lXBit := 128;
                  lXOffset := 0;
                  repeat
                      If (sbyte And lXBit) = lXBit Then
                      begin;
                          P1[charstart+lXOffset] := fcolor;
                          P2[charstart+lXOffset] := fcolor;
                          Inc(lXOffset);
                          P1[charstart+lXOffset] := fcolor;
                          P2[charstart+lXOffset] := fcolor;
                          Inc(lXOffset);
                      end
                      Else
                      begin
                          P1[charstart+lXOffset] := bcolor;
                          P2[charstart+lXOffset] := bcolor;
                          Inc(lXOffset);
                          P1[charstart+lXOffset] := bcolor;
                          P2[charstart+lXOffset] := bcolor;
                          Inc(lXOffset);
                      end;
                      lXBit := lXBit shr 1;
                  Until lXBit = 0;
              End;
              ScrnLines[lne, X] := False;
          end;
          ScrnLines[lne, 32] := False;
      End;
  end;
  frmMainWnd.DXPaintBoxSpecScreen.Paint;
  ScrnNeedRepaint := False;
End;


procedure screenPaint3X;
    var
    lne : integer; X : integer;
    sbyte : integer; abyte : integer; fcolor : integer; bcolor : integer;
    lXBit : integer; lXOffset : integer;
    charstart : integer;
    P1:^TByteArray;
    P2:^TByteArray;
    P3:^TByteArray;
begin

  If ScrnNeedRepaint = False Then
    Exit;

  For lne := 0 To 191 do
  begin
      If ScrnLines[lne, 32] = True Then
      begin
          P1:=frmMainWnd.DXPaintBoxSpecScreen.DIB.ScanLine[lne * 3];
          P2:=frmMainWnd.DXPaintBoxSpecScreen.DIB.ScanLine[lne * 3 + 1];
          P3:=frmMainWnd.DXPaintBoxSpecScreen.DIB.ScanLine[lne * 3 + 2];
          For X := 0 To 31 do
          begin
              If ScrnLines[lne, X] = True Then
              begin
                  sbyte := gRAMPage[glUseScreen, glScreenMem[lne, X]];
                  abyte := gRAMPage[glUseScreen, (6144 + (lne shr 3) shl 5 + X)];

                  charStart := X * 24;
                  If (abyte And 64) = 64 Then
                  begin
                     fcolor := (abyte And 7) + 8;
                     bcolor := ((abyte And 56) shr 3) + 8;
                  end
                  Else
                  begin
                     fcolor := (abyte And 7);
                     bcolor := ((abyte And 56) shr 3);
                  End;
                  If ((abyte And 128) = 128) And (bFlashInverse) Then
                  begin
                      // Swap fore- and back-colours
                      lXBit := fcolor;
                      fcolor := bcolor;
                      bcolor := lXBit;
                  End;

                  lXBit := 128;
                  lXOffset := 0;
                  repeat
                      If (sbyte And lXBit) = lXBit Then
                      begin;
                          P1[charstart+lXOffset] := fcolor;
                          P2[charstart+lXOffset] := fcolor;
                          P3[charstart+lXOffset] := fcolor;
                          Inc(lXOffset);
                          P1[charstart+lXOffset] := fcolor;
                          P2[charstart+lXOffset] := fcolor;
                          P3[charstart+lXOffset] := fcolor;
                          Inc(lXOffset);
                          P1[charstart+lXOffset] := fcolor;
                          P2[charstart+lXOffset] := fcolor;
                          P3[charstart+lXOffset] := fcolor;
                          Inc(lXOffset);
                      end
                      Else
                      begin
                          P1[charstart+lXOffset] := bcolor;
                          P2[charstart+lXOffset] := bcolor;
                          P3[charstart+lXOffset] := bcolor;
                          Inc(lXOffset);
                          P1[charstart+lXOffset] := bcolor;
                          P2[charstart+lXOffset] := bcolor;
                          P3[charstart+lXOffset] := bcolor;
                          Inc(lXOffset);
                          P1[charstart+lXOffset] := bcolor;
                          P2[charstart+lXOffset] := bcolor;
                          P3[charstart+lXOffset] := bcolor;
                          Inc(lXOffset);
                      end;
                      lXBit := lXBit shr 1;
                  Until lXBit = 0;
              End;
              ScrnLines[lne, X] := False;
          end;
          ScrnLines[lne, 32] := False;
      End;
  end;
  frmMainWnd.DXPaintBoxSpecScreen.Paint;
  ScrnNeedRepaint := False;
End;




procedure screenPaint;
begin
  if (glDisplayMultiplier=1) then
    screenPaint1X
  else if (glDisplayMultiplier=2) then
    screenPaint2X
  else if (glDisplayMultiplier=3) then
    screenPaint3X
  else
    screenPaint1X
end;

end.
