unit options;
{ /*******************************************************************************
'   Options.pas within DelphiSpec.dpr
'
'   Various emulation settings (emulated model, speed, sound)
'   are changed here
'
'   Author: Chris Cowley <ccowley@grok.co.uk>
'
'   Copyright (C)1999-2000 Grok Developments Ltd.
'   http://www.grok.co.uk/
'
'   This program is free software; you can redistribute it and/or
'   modify it under the terms of the GNU General Public License
'   as published by the Free Software Foundation; either version 2
'   of the License, or (at your option) any later version.
'   This program is distributed in the hope that it will be useful,
'   but WITHOUT ANY WARRANTY; without even the implied warranty of
'   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'   GNU General Public License for more details.
'
'   You should have received a copy of the GNU General Public License
'   along with this program; if not, write to the Free Software
'   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' *******************************************************************************/}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TfrmOptions = class(TForm)
    lblStatic: TLabel;
    cboModel: TComboBox;
    chkSound: TCheckBox;
    grpEmulationSpeed: TRadioGroup;
    btnOK: TButton;
    btnCancel: TButton;
    procedure FormShow(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmOptions: TfrmOptions;

implementation

uses modMain;

{$R *.DFM}

procedure TfrmOptions.FormShow(Sender: TObject);
begin
    Case glInterruptDelay of
      40: // 40ms delay = 50% Spectrum speed
        grpEmulationSpeed.ItemIndex := 0;
      20: //20ms delay = 100% Spectrum speed
        grpEmulationSpeed.ItemIndex := 1;
      10: //10ms delay = 200% Spectrum speed
        grpEmulationSpeed.ItemIndex := 2;
       0: // 0 delay = run as fast as possible
        grpEmulationSpeed.ItemIndex := 3;
       else
        grpEmulationSpeed.ItemIndex := 1;
    End;

    Case glEmulatedModel of
      0:  // 48K
        cboModel.ItemIndex := 0;
      1:  //128K
        cboModel.ItemIndex := 1;
      2:  //+2
        cboModel.ItemIndex := 2;
      else
        cboModel.ItemIndex := 1;
    End;

    {$IFDEF USE_SOUND}
      chkSound.Enabled := True;
      If gbSoundEnabled Then chkSound.checked := true Else chkSound.checked := false;
    {$ELSE}
        chkSound.Enabled := False;
    {$ENDIF}
end;

procedure TfrmOptions.btnOKClick(Sender: TObject);
  var dum: integer;
begin
    If grpEmulationSpeed.ItemIndex = 0 Then
        glInterruptDelay := 40
    Else If grpEmulationSpeed.ItemIndex = 1 Then
        glInterruptDelay := 20
    Else If grpEmulationSpeed.ItemIndex = 2 Then
        glInterruptDelay := 10
    Else If grpEmulationSpeed.ItemIndex = 3 Then
        glInterruptDelay := 0;
    SaveSetting('InterruptDelay', IntToStr(glInterruptDelay));
    
    glEmulatedModel := cboModel.ItemIndex;
    SaveSetting('EmulatedModel', IntToStr(glEmulatedModel));

    {$IFDEF USE_SOUND}
    if (chkSound.Checked) then dum:=1 else dum:= 0;
    SaveSetting('SoundEnabled', IntToStr(dum));
    If gbSoundEnabled Then
    begin
      If chkSound.checked = false Then
        CloseWaveOut;
    end
    Else
    begin
      If chkSound.checked = true Then
        gbSoundEnabled := InitializeWaveOut;
    End;
    {$ENDIF}
end;

end.
